#
# Xsigo Xen utilities
#
# Author: David Allison
#
# Copyright (C) 2007 Xsigo Systems Inc.  All rights reserved
#

import os
import sys

# uses functions from the xen.xm module

from xen.xm.main import xm_network_attach
from xen.xm.main import xm_network_detach
from xen.xm.main import xm_block_attach
from xen.xm.main import xm_block_detach
from xen.xm.main import arg_check
from xen.xm.main import getDomains
from xen.xend import sxp


import xen.xm.create 

# xsigo utilities

import xsigoutils

def help():
    print "Xsigo XEN support utilities"
    print ""
    print "Commands:"
    print "    xgxen vnic-list"
    print "    xgxen vnic-attach <dom> <vnic>"
    print "    xgxen vnic-detach <dom> <vnic>"
    print "    xgxen vhba-list"
    print "    xgxen vhba-attach <dom> <vhba> <devroot> <mode>"
    print "    xgxen vhba-detach <dom> <vhba>"
    print "    xgxen set-macs <bridge>"
    print "    xgxen get-macs <bridge>"
    print "    xgxen finish-restore <vmname>"
    print "    xgxen create configfile [-hvm] [bridge:vnic,...] [vhba,...]"

def xg_vnic_list(args):
    xsigoutils.list_vnics()

def xg_vnic_attach(args):
    arg_check(args, 'vnic-attach', 2)

    domname = args[0]
    vnic = args[1]

    bridge = xsigoutils.make_vnic_bridge (vnic)

    xm_network_attach ([domname, 'bridge=' + bridge])

    vnicmac = xsigoutils.get_vnic_mac (vnic)
    print "vnic: " + vnicmac

    doms = getDomains("")
    dom = xsigoutils.find_domain (doms, domname)

    mac = xsigoutils.get_mac (dom, bridge)
    print "guest: " + mac

    # program the l2 table in the Xsigo iocard
    if mac != "":
        e = os.system ("ifconfig " + vnic + " hw ether " + mac)
        if os.WIFEXITED(e):
            s = os.WEXITSTATUS(e)
            if s != 0:
                raise Exception ("Failed to set mac address for virtual NIC")

        e = os.system ("ifconfig " + vnic + " hw ether " + vnicmac)
        if os.WIFEXITED(e):
            s = os.WEXITSTATUS(e)
            if s != 0:
                raise Exception ("Failed to set mac address for virtual NIC")

    os.system ("ifconfig " + vnic + " up")
    os.system ("ifconfig " + bridge + " up")

def xg_vnic_detach(args):
    """detach a vnic from a vm"""
    arg_check(args, 'vnic-detach', 2)

    doms = getDomains("")
    domname = xsigoutils.find_domain_name (doms, args[0])
    vnic = args[1]

    bridge, newbridge, addvnic = xsigoutils.get_bridge_name (vnic)
    ifaces = xsigoutils.get_bridge_interfaces (bridge)

    # the vnic may be attached to multiple interfaces in the domain.  Detach them all
    for iface in ifaces:
        if "vif" in iface:
            dot = iface.find('.')
            domid = iface[3:dot]
            devid = iface[dot+1:]
            name = xsigoutils.find_domain_name (doms, domid)
            if name == domname:
                xm_network_detach ([name, devid])

def xg_vhba_list(args):
    xsigoutils.list_vhbas()

def xg_vhba_attach(args):
    arg_check(args, 'vhba-attach', 4)

    domname = args[0]
    vhba = args[1]
    root = args[2]
    mode = args[3]

    letter = 0

    vdev = root
    pdevs = xsigoutils.get_vhba_devices (vhba)
    for pdev in pdevs: 
        # remove /dev/ from device if present
        if pdev.startswith ("/dev/"):
            pdev = pdev[5:]

        blockargs = [domname, "phy:" + pdev,  vdev + xsigoutils.base26(letter), mode]
        try:
            xm_block_attach (blockargs)
        except Exception,e: 
            # ignore exception if it is from destroyDevice
            if 'destroyDevice' in str(e):
                pass
            raise Exception, e
        letter = letter + 1


def xg_vhba_detach(args): 
    """detach a vhba from a vm"""
    arg_check(args, 'vhba-detach', 2)
    domname = args[0]
    vhba = args[1]

    vdevs = xsigoutils.get_vhba_info (vhba)
    for vdev in vdevs:
        xm_block_detach ([domname, vdev])


# I wanted to declare this inside the xg_create function but it won't work
# it appears that python doesn't support lexical scopes even though it 
# is supposed to.  A proper language like Aikido would make this much easier and cleaner!!!
xg_comma = False
xg_vif = 'vif='

def xg_create (args):
    from xen.util import Brctl

    """ create a vm allowing bridge and vhba args"""
    arg_check(args, 'create', 1, 4)
    configfile = args[0]
    bridgelist = ""
    vhbalist = ""
    ioemu = False
    for arg in args[1:]:
        if arg == "-hvm":
            ioemu = True
        else:
            if bridgelist == "":
                bridgelist = arg
            else:
                vhbalist = arg

    bridges = []
    vhbas = []
    create_args = ['xm', configfile]
    
    def append (s):
        global xg_comma   # what a horrible hack!
        global xg_vif
        if xg_comma:
            xg_vif = xg_vif + ','     # and there is no append for strings!!!
        xg_vif = xg_vif + s
        xg_comma = True

    if ioemu:
        append ('type=ioemu')

    # bridges is a comma-separated list of bridges to attach to the domain
    if not bridgelist == "":
        bridges = bridgelist.split(',')

    for bridge in bridges:
        if not bridge == "":
            mac = ''
            # @ separated tuple lf bridge;mac;vnic
            if '@' in bridge:
                (bridge,mac,vnic) = bridge.split('@')
                try:
                    Brctl.bridge_create(bridge)
                except Exception, e:
                    pass
                if vnic != '':
                    Brctl.vif_bridge_add ({'bridge':bridge, 'vif': vnic})
            else:
                raise Exception, 'bridge syntax error, need brige@mac@vnic format'
            append ('bridge=%s' % bridge)
            if mac != '':
                append ('mac=%s' % mac)

            create_args.append (xg_vif)

    if not vhbalist == "":
        vhbas = vhbalist.split(',')

    letter = 3
    for vhba in vhbas:
        if not vhba == "":
            devs = xsigoutils.get_vhba_info (vhba)
            for dev in devs:
                create_args.append ('disk=phy:%s,xvd%s,w' % (dev,xsigoutils.base26(letter)))
                letter = letter + 1

    # now create the domain
    
    xen.xm.create.main (create_args)

    # program the l2 table (will go away if/when mac learning is in place)
    for bridge in bridges:
        if not bridge == "":
            if '@' in bridge:
                (bridge,mac,vnic) = bridge.split('@')
            xg_set_macs ([bridge])
            os.system ("ifconfig " + bridge + " up")

def xg_finish_restore(args):
    """finish the restoration of a vm"""
    arg_check(args, 'finish-restore', 1)
    domainname = args[0]
    # find the bridges attached to the guest
    doms = xsigoutils.getDomains('')
    dom = xsigoutils.find_domain (doms, domainname)
    domid = sxp.child_value(dom, 'domid', 'unknown')

    bridges = xsigoutils.get_bridges()

    # go through all the bridges attached to the domain and set the macs
    # for the interface
    for bridge in bridges:
        ifaces = xsigoutils.get_bridge_interfaces (bridge.name)
        for iface in ifaces:
            if 'vif' in iface:
                brdomid = ""
                ch = 3
                while iface[ch] != '.':
                    brdomid = brdomid + iface[ch]
                    ch = ch + 1
                if int(brdomid) == domid:
                    xg_set_macs ([bridge.name])


def xg_set_macs (args):
    """ set the mac addresses for a vnic attached to a bridge"""
    arg_check(args, 'set_macs', 1)
    bridge = args[0]
    interfaces = xsigoutils.get_bridge_interfaces (bridge)
    doms = xsigoutils.getDomains('')


    vnics = []

    for iface in interfaces:
        if xsigoutils.is_vnic(iface):
            vnics.append (iface)

    for iface in interfaces:
        if 'vif' in iface:
            ch = 3
            domid = ""
            while iface[ch] != '.':
                domid = domid + iface[ch]
                ch = ch + 1
            domain = xsigoutils.find_domain (doms, domid)
            mac = xsigoutils.get_mac (domain, bridge)

            # set the mac address from the domain in all the vnics in the bridge
            for vnic in vnics:
                print 'Programming mac ' + mac + ' through vnic ' + vnic
                vnicmac = xsigoutils.get_vnic_mac (vnic)
                # program the l2 table in the Xsigo iocard
                e = os.system ("ifconfig " + vnic + " hw ether " + mac)
                if os.WIFEXITED(e):
                    s = os.WEXITSTATUS(e)
                    if s != 0:
                        raise Exception ("Failed to set mac address for virtual NIC")
    
                e = os.system ("ifconfig " + vnic + " hw ether " + vnicmac)
                if os.WIFEXITED(e):
                    s = os.WEXITSTATUS(e)
                    if s != 0:
                        raise Exception ("Failed to set mac address for virtual NIC")

             
def xg_get_macs (args):
    """ get the mac addresses for all guests attached to a bridge"""
    arg_check(args, 'get_macs', 1)
    bridge = args[0]
    interfaces = xsigoutils.get_bridge_interfaces (bridge)
    doms = xsigoutils.getDomains('')

    for iface in interfaces:
        if 'vif' in iface:
            ch = 3
            domid = ""
            while iface[ch] != '.':
                domid = domid + iface[ch]
                ch = ch + 1
            domain = xsigoutils.find_domain (doms, domid)
            mac = xsigoutils.get_mac (domain, bridge)
            domainname = xsigoutils.find_domain_name (doms, domid)
            print domainname, mac
             

commands = [
    ['vnic-attach', xg_vnic_attach],
    ['vnic-list', xg_vnic_list],
    ['vnic-detach', xg_vnic_detach],
    ['vhba-attach', xg_vhba_attach],
    ['vhba-list', xg_vhba_list],
    ['vhba-detach', xg_vhba_detach],
    ['create', xg_create],
    ['set-macs', xg_set_macs],
    ['get-macs', xg_get_macs],
    ['finish-restore', xg_finish_restore],
    ]

def get_command (name):
    for cmd in commands:
        if name == cmd[0]:
            return cmd[1]
    raise Exception, 'Invalid xsigo xen command'

def main(argv):
    if len(argv) < 2:
        help()
        return

    args = argv[2:]
    
    try:
        cmd = get_command (argv[1])
        cmd(args)
    except Exception,e:
        print str(e)
        help()
        sys.exit(1)

if __name__ == "__main__":
    main(sys.argv)
