#!/bin/bash
# Copyright (c) 2006-2008 Xsigo Systems Inc.  All rights reserved.
#
#  This software is available to you under a choice of one of two
#  licenses.  You may choose to be licensed under the terms of the GNU
#  General Public License (GPL) Version 2, available from the file
#  COPYING in the main directory of this source tree, or the
#  OpenIB.org BSD license below:
# 
#      Redistribution and use in source and binary forms, with or
#      without modification, are permitted provided that the following
#      conditions are met:
# 
#       - Redistributions of source code must retain the above
#         copyright notice, this list of conditions and the following
#         disclaimer.
# 
#       - Redistributions in binary form must reproduce the above
#         copyright notice, this list of conditions and the following
#         disclaimer in the documentation and/or other materials
#         provided with the distribution.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
#  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
#  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
#  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
# 
# 

if [ -z $BASEDIR ]; then
	BASEDIR=/
fi

errno=0
LOGFILE=$BASEDIR/var/log/xsigo/xsadmdsh.log
export  PATH=$PATH:/usr/bin:/usr/sbin:/sbin
DHCPLIST_FILE="/var/run/dhcplist.xsigod"

function log_print()
{
	echo "`date '+%b %d %T'`" $* >> $LOGFILE
	return
}

function log_init()
{
	LOGFILE=$BASEDIR/var/log/xsigo/xsadmdsh.log
	export LOGFILE
	mkdir -p  /var/log/xsigo
	touch $LOGFILE
	log_print "Starting xsmaster script"
	cp $BASEDIR/opt/xsigo/xsigod.conf /etc/logrotate.d/xsigod
	touch $DHCPLIST_FILE
	logrotate /etc/logrotate.conf
}

function xsmaster_main()
{
case "$1" in
	"loginit")
		log_init
		exit 0
		;;
	"init")
		log_init
		modprobe bonding miimon=200
		#
		# Make sure bonding sysfs is created
		#
		for i in {1..10}
		do
			if [ -f /sys/class/net/bonding_masters ]; then
				break
			fi
			sleep 1
		done
		exit 0
		;;
	"uninit")
		log_print "Exiting xsmaster script"
		exit 0
		;;
	"hostname")
		HOSTNAME=`hostname`
		echo $HOSTNAME > /proc/driver/xscore/info
		exit 0
		;;
	"systemid")
		SYSTEM_UUID=$(dmidecode -t system 2>/dev/null|grep UUID:|cut -f2 -d:|sed -e 's/-//g')
		if echo $SYSTEM_UUID | grep -i Settable  > /dev/null
		then
			SYSTEM_UUID=`hostid`
		else
			SYSTEM_UUID=$(echo $SYSTEM_UUID | tr [:upper:] [:lower:] 2>/dev/null)
		fi
		echo $SYSTEM_UUID > /proc/driver/xscore/systemid
		exit 0
		;;

	"vnic")
		log_print "Processing vnic messages"
		command=$BASEDIR/opt/xsigo/bin/xs$1.sh
		shift
		args=$*
		log_print "Executing command $command $args"
		$command $args >> $LOGFILE 2>&1
                errno=$?
                if [ $errno != 0 ]; then
                        log_print "Failed, with err-no $errno"
		else
			log_print "Passed"
                fi
                exit $errno
		;;

	"vnic-noprint")
		command=$BASEDIR/opt/xsigo/bin/xsvnic.sh
		shift
		args=$*
		$command $args >> $LOGFILE 2>&1
                errno=$?
                if [ $errno != 0 ]; then
                        log_print "Failed, with err-no $errno"
                fi
                exit $errno
		;;

	"vhba")
		log_print "Processing vhba messages"
 		command=$BASEDIR/opt/xsigo/bin/xs$1.sh
                shift
                args=$*
		log_print "Executing command $command $args"
                $command $args >> $LOGFILE 2>&1
                errno=$?
                if [ $errno != 0 ]; then
                        log_print "Failed, with err-no $errno"
		else
			log_print "Passed"
                fi
                exit $errno
		;;

	*)
		log_print " <$*> is not a supported command"
		exit 127
		;;
esac
}

#Master script execution  starts from here
xsmaster_main $*
