#!/bin/bash 
# Copyright (c) 2006-2008 Xsigo Systems Inc.  All rights reserved.
#
#  This software is available to you under a choice of one of two
#  licenses.  You may choose to be licensed under the terms of the GNU
#  General Public License (GPL) Version 2, available from the file
#  COPYING in the main directory of this source tree, or the
#  OpenIB.org BSD license below:
# 
#      Redistribution and use in source and binary forms, with or
#      without modification, are permitted provided that the following
#      conditions are met:
# 
#       - Redistributions of source code must retain the above
#         copyright notice, this list of conditions and the following
#         disclaimer.
# 
#       - Redistributions in binary form must reproduce the above
#         copyright notice, this list of conditions and the following
#         disclaimer in the documentation and/or other materials
#         provided with the distribution.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
#  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
#  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
#  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
# 

export  PATH=$PATH:/usr/bin:/usr/sbin:/sbin
DHCPLIST_FILE="/var/run/dhcplist.xsigod"
touch $DHCPLIST_FILE
errno=0


# Intermediate temporary file used as reference to filter out removed HM vlans

#Error codes
XSADM_MODPROBE_ERROR=1
XSADM_IFUP_ERROR=2
XSADM_VCONFIG_ERROR=3
XSADM_IPCFG_ERROR_DHCP=4
XSADM_IPCFG_ERROR_STATIC=5

function xsadmd_update_lease()
{
	interface=$1
	# optionally  dont do an explicit expiration of lease 
	if [ ! "$XSIGO_DHCP_LEASE_DONT_EXPIRE" ] ; then
		#terminate the lease with -r option with a single attempt 
		dhclient -r -1 $interface
		result=$?
	fi
	return $result
	
}

function get_bond_name()
{
	bname=`dmesg | grep "divert: allocating divert_blk for bond" | tail -1 | awk '{ print $5 }'`
	echo "$bname"
}

function xsadmd_execute_dhcp()
{
	interface=$1
	DHC_FILE=/sbin/dhclient
	if [ -f  $DHC_FILE ] ; then
		PID_FILE=/var/run/dhclient-$interface.pid
		touch $PID_FILE
		$DHC_FILE -pf $PID_FILE -nw $interface
	else
		dhcpcd -C -K -N $interface -c \
			/etc/sysconfig/network/scripts/dhcpcd-hook
	fi
	grep $interface $DHCPLIST_FILE > /dev/null
	if [ $? != 0 ] ; then
		echo $interface >> $DHCPLIST_FILE
	fi
	return 0
}

function clear_vnic_dhclient()
{
	interface=$1
	DH_PID_FILE="/var/run/dhclient-$interface.pid"
	if [ -f $DH_PID_FILE ] ; then
		VNIC_PID=`cat $DH_PID_FILE`
		xsadmd_update_lease $interface
		if [ "$VNIC_PID" ] ; then
			kill $VNIC_PID
		fi
		rm -f $DH_PID_FILE
		sed '/'$interface'/d' $DHCPLIST_FILE > /tmp/dhcplist.xsigod$$
		cp /tmp/dhcplist.xsigod$$ $DHCPLIST_FILE
		rm -f /tmp/dhcplist.xsigod$$
	fi
	return 0
}
		

function xsvnic_main()
{
	case "$1" in

	"oper_up")
		interface=$2
		ifconfig $interface | grep  RUNNING
		exit $?
		;;

	"havnic-remove")
		BOND=$2
		SLAVE=$3
		if [ -f /sys/class/net/bonding_masters ]; then
			#
			# If this is the last one delete the bond
			#
			grep $BOND /sys/class/net/$BOND/bonding/slaves
			if [ $? != 0 ]; then
				echo "-$BOND" > /sys/class/net/bonding_masters
			fi
		elif [ -e /etc/redhat-release ] &&  grep "Nahant" /etc/redhat-release > /dev/null
		then
			if grep "Active Slave: None" /proc/net/bonding/$BOND > /dev/null
			then
				rmmod $BOND
			fi
		fi
                exit 0
		;;

	"havnic-install")
		BOND=$2
		SLAVE=$3
		XPRIMARY=$4
		PRIMARY=$5

		#
		# Make sure slave interface is UP
		#
		sysctl net.ipv6.conf.$BOND.accept_dad=0
		sysctl net.ipv6.conf.$SLAVE.accept_dad=0
		for i in {1..10}
		do
			if ifconfig $SLAVE > /dev/null 2>&1
			then
				break
			fi
			sleep 1
		done
		if [ -f /sys/class/net/bonding_masters ]; then
			if [ ! -d /sys/class/net/$BOND ]; then
				echo "+$BOND" > /sys/class/net/bonding_masters
				echo 1 > /sys/class/net/$BOND/bonding/mode
				echo 100 >/sys/class/net/$BOND/bonding/miimon
				echo 1 > /sys/class/net/$BOND/bonding/use_carrier
				MTU=`cat /sys/class/net/$SLAVE/mtu`
				ifconfig $BOND mtu $MTU up
			fi
			ifconfig $SLAVE 0.0.0.0 down
			echo "+$SLAVE" > /sys/class/net/$BOND/bonding/slaves
			if [ "$PRIMARY" ] && [ "$PRIMARY" == "$SLAVE" ]
			then
				echo $PRIMARY > /sys/class/net/$BOND/bonding/primary
			fi
			if [ "$XPRIMARY" ] && [ "$XPRIMARY" == "$SLAVE" ]
			then
				for j in {1..30}
				do 
					if [ "`cat /sys/class/net/$XPRIMARY/carrier`" == 1 ]
					then 
						break					
					fi
					sleep 1
				done
				echo $XPRIMARY > /sys/class/net/$BOND/bonding/active_slave
			fi
		elif [ -e /etc/redhat-release ] &&  grep "Nahant" /etc/redhat-release > /dev/null
		then
			if [ ! -d /sys/class/net/$BOND ]; then
				if [ "$PRIMARY" ]; then
					modprobe bonding -o $BOND miimon=200 mode=1 use_carrier=1 primary=$PRIMARY
				else
					modprobe bonding -o $BOND miimon=200 mode=1 use_carrier=1
				fi
				MTU=`cat /sys/class/net/$SLAVE/mtu`
				ifconfig $SLAVE 0.0.0.0 down
				ABOND=`get_bond_name`
				ip link set dev $ABOND name $BOND
				ifconfig $BOND mtu $MTU up
			fi
			ifconfig $SLAVE 0.0.0.0 down
			ifenslave $BOND $SLAVE
		fi
                exit 0
		;;

	"install")
		interface=$2
		sysctl net.ipv6.conf.$interface.accept_dad=0
		ifconfig $interface up
		exit $?
		;;

	"link_up")
                exit 0
		;;

	"link_down")
		exit 0
		;;

	"update")
		interface=$2
                ifconfig $interface $3 netmask $4 up
		result=$?
		exit $result
		;;
	"ha-check")
		VNIC_DIR=/proc/driver/xsvnic/devices
		if [ -d $VNIC_DIR ]; then
			cd $VNIC_DIR
			for vnic in `ls $VNIC_DIR`
			do
				HA_STATE=`cat $vnic/$vnic | grep "HA flags:" | cut -f 2 -d ":"`
				if [ $HA_STATE != "None" ]; then
				   #HA VNICS
				   TMP_PROC=`cat /proc/driver/xsvnic/devices/${vnic}/${vnic} |grep "HA Active State" | cut -f 2 -d ":" `
				   BOND_STATE_PROC=`echo $TMP_PROC | tr -d ' ' `
				   BOND_NAME=`echo $vnic | sed -e s/"_.*"//g`
				   if [ -f  "/proc/net/bonding/$BOND_NAME" ]; then
					ACTIVE_SLAVE=`cat /proc/net/bonding/$BOND_NAME | grep "Currently Active Slave" | cut -f 2 -d ":"`
					if [ $BOND_STATE_PROC == "slaveactive" ]; then
						if [ $ACTIVE_SLAVE != $vnic ]; then
							echo "slave backup" > /proc/driver/xsvnic/devices/${vnic}/${vnic}
						fi
					elif [ $BOND_STATE_PROC == "slavebackup" ]; then
						if [ $ACTIVE_SLAVE == $vnic ]; then
							echo "slave active" > /proc/driver/xsvnic/devices/${vnic}/${vnic}
						fi
					fi
			          fi
				fi
			done
		fi
		exit 0
		;;
	"clear_dhclient")
		if [ ! -f $DHCPLIST_FILE ] ; then
			exit 0
		fi
		DHCPLIST_FILE_ORG="/var/run/dhcplist_org.xsigo"
		cp $DHCPLIST_FILE $DHCPLIST_FILE_ORG
		while read line
		do
		   act_interface=$(echo $line | awk '{ print $1 }')
		   # Get the vnic name if this is a vlan name 
		   interface=$(echo $act_interface | cut -f1 -d ."" )
		   PROC_FILE=/proc/driver/xsvnic/devices/$interface/$interface
		   if [ ! -f "$PROC_FILE" ] ; then
			clear_vnic_dhclient $act_interface
	           fi 
		done < $DHCPLIST_FILE_ORG

		rm $DHCPLIST_FILE_ORG
		exit $?
		;;

	"clear_vnic_dhclient")
		interface=$2
		clear_vnic_dhclient $interface
                exit 0
		;;

	"dhcp")
		interface=$2
		xsadmd_execute_dhcp $interface
		exit 0
		;;

	"vlan")
		vlan_instance=$3
		interface=$4
		vlan_interface=$interface.$vlan_instance
		result=0
		modprobe 8021q
		result=$?
		if [ $result != 0 ] ; then
			exit $XSADM_MODPROBE_ERROR
		fi
		ifconfig  $interface up
		result=$?
		if [ $result != 0 ] ; then
			exit $XSADM_IFUP_ERROR

		fi
		if [ -f /sbin/vconfig ] ; then
			VCONFIG=/sbin/vconfig
		else
			#it's not in /sbin in SLES! 
	                VCONFIG= /usr/sbin/vconfig
		fi
		

		case "$2" in
			"add")
				result=0
				if [ ! -f /proc/net/vlan/$vlan_interface ] ; then
				     $VCONFIG add $interface $vlan_instance
				     result=$?
				fi
				if [ $result != 0 ] ;then
					exit $XSADM_VCONFIG_ERROR
				fi
				exit 0
				;;

			"delete")
				$VCONFIG rem $vlan_interface 
				result=$?
				exit $result
				;;

			"dhcp")
				xsadmd_execute_dhcp $vlan_interface
				result=$?
				exit 0
				;;

			"static")
				ifconfig $vlan_interface $5 netmask $6 up
				result=$?
				if [ $result != 0 ] ;then
					exit $XSADM_IPCFG_ERROR_STATIC
				fi
				exit 0
				;;

			*)
                		exit 127
                		;;
		esac
		;;

	"cnic_cfg")
		interface=$2
		ip=$3
		if [ "$ip" != 0 ]; then
			ifconfig $interface $ip netmask $4 up
		else
			ifconfig $interface down
		fi
		exit 0
		;;

	"gateway")
		case "$2" in
			"add")
				netstat -r | grep default | grep "$3"
				if [ $? != 0 ]; then	
					route add default  gw $3
					exit $?
				fi
				exit 0
				;;

			"delete")
				route delete default  gw $3
				exit 0
				;;

			"dns")
				case "$3" in
					"add_ip")
						cat /etc/resolv.conf | grep $4
						if [ $? != 0 ]; then	
							echo "nameserver $4" >> /etc/resolv.conf
						fi
						exit $?
						;;

					"delete_ip")
						sed '/nameserver '$4'/d' /etc/resolv.conf > /tmp/resolv.$$
						mv /tmp/resolv.$$ /etc/resolv.conf
						exit $?
						;;

					"add_dmn_name")
						cat /etc/resolv.conf | grep $4
						if [ $? != 0 ]; then
							sed '/search/ s/$/ '$4'/' /etc/resolv.conf > /tmp/resolv.$$
							mv /tmp/resolv.$$ /etc/resolv.conf
						fi
						exit $?
						;;

					"delete_dmn_name")
						sed '/ '$4'/s///g' /etc/resolv.conf > /tmp/resolv.$$
						mv /tmp/resolv.$$ /etc/resolv.conf
						exit $?
						;;

					*)
                				exit 127
               				;;
				esac
				;;

			*)
				exit 127
      				;;
		esac
		;;

	*)
		exit 127
		;;
esac
}

#Scripts starts here
xsvnic_main $*
